class InvestorProfileForm
  include ActiveModel::Model
  include Virtus.model

  attr_accessor :member, :investor_account, :bank_account
  attribute :model_id, Integer

  AggregateRoot = Member

  # Member fields
  attribute :name, String
  attribute :nationality, String
  attribute :id_number, String
  attribute :date_of_birth, Date
  attribute :phone_number, String
  attribute :address1, String
  attribute :address2, String
  attribute :city, String
  attribute :postal_code, String
  attribute :country_of_residence, String
  attribute :id_document, String
  attribute :id_document_additional, String
  attribute :proof_of_residence, String

  # Bank account fields
  attribute :account_holder, String
  attribute :account_number, String
  attribute :account_type, String
  attribute :bank_name, String
  attribute :bank_address, String
  attribute :bank_country, String
  attribute :swift_code, String

  class << self
    def find(id)
      member = AggregateRoot.find id
      investor_account = member.investor_account
      bank_account = investor_account.bank_accounts.first
      attrs = member.attributes.merge(investor_account.attributes).merge(bank_account.attributes)
      form = self.new attrs
      form.set_model id
      return form
    end

    def update(params)
      if params[:model_id].blank?
        raise 'Model ID not found'
      end
      form = self.new sanitized_params(params)
      form.set_model params[:model_id]
      form.persist(params)
      return form
    end

    def sanitized_params(params)
      params[:date_of_birth] = Date.parse("#{params['date_of_birth(3i)']}-#{params['date_of_birth(2i)']}-#{params['date_of_birth(1i)']}")
      return params
    end
  end

  def set_model(member_id)
    @model_id = (member_id.respond_to? :id)? member_id.id : member_id
    @member = AggregateRoot.find member_id
    @investor_account = @member.investor_account
    @bank_account = @investor_account.bank_accounts.first
  end

  def persist(params)
    params.each do |k,v|
      setter_method = "#{k}="
      if @member.respond_to? setter_method
        @member.send setter_method, v
      end
      if @investor_account.respond_to? setter_method
        @investor_account.send setter_method, v
      end
      if @bank_account.respond_to? setter_method
        @bank_account.send setter_method, v
      end
    end
    @member.save!
    @investor_account.save!
    @bank_account.save!
  end

  module Validator
    extend ActiveSupport::Concern

    included do
      validates_presence_of :name,
                            :id_number, :date_of_birth, :phone_number,
                            :address1, :city, :postal_code,
                            :country_of_residence, :nationality,
                            :account_holder, :account_number, :account_type,
                            :bank_country, :bank_name

      validates_presence_of :id_document, if: :id_document_blank?
      validates_presence_of :id_document_additional, if: :is_passport_blank_and_not_local_region?
      validate :age_must_be_above_18
      validate :name_and_account_holder_must_be_the_same
    end

    def age_must_be_above_18
      too_young = date_of_birth.present? && TimeDifference.between(date_of_birth, Date.today).in_years.to_i < 18
      errors.add(:date_of_birth, I18n.t('investor_form.must_be_older_than_18')) if too_young
    end

    def name_and_account_holder_must_be_the_same
      if name.to_s != account_holder.to_s
        msg = I18n.t('investor_form.name_and_account_holder_must_be_the_same')
        errors.add(:name, msg)
        errors.add(:account_holder, msg)
      end
    end

    def id_document_blank?
      @member.id_document.blank?
    end

    def is_passport_blank_and_not_local_region?
      is_local_region = %w(Singapore Indonesia).include? nationality
      !is_local_region && @member.id_document_additional.blank?
    end
  end
  include Validator
end